# -*- coding: utf-8 -*-
import html2text
import io
import json
import logging
import os
import re
import werkzeug

from PyPDF2 import PdfFileWriter, PdfFileReader

from reportlab.pdfgen import canvas
from reportlab.lib.units import cm

from odoo import tools
from odoo.addons.saas_worker.compat import route, http, request

_logger = logging.getLogger(__name__)


class OdooMisc(http.Controller):
    @route(['/app/localization', '/page/accounting/<string:l10n_app>', '/app/accounting/<string:l10n_app>'],
           auth='public', website=True, multilang=True, sitemap=True, readonly=True)
    def l10n_app_geoip_redirect(self, l10n_app=None, **kw):
        supported_l10n = {
            'AR': 'argentina',
            'CL': 'chile',
            'CO': 'colombia',
            'PE': 'peru',
            'MX': 'mexico',
        }

        l10n_code = None

        # Retro-compatibility for the /page/accounting/<string:l10n_code> route
        if l10n_app and len(l10n_app) >= 7:
            l10n_code = l10n_app[5:7].upper()
        else:
            l10n_code = request.geoip.country_code

            # Get the country_code from the partner instead if the user is logged in
            if request.website.partner_id != request.env.user.partner_id:
                partner_id = request.env.user.partner_id
                # Allow localization spoofing for employees only
                l10n_code = l10n_code if request.env.user.has_group('base.group_user') else partner_id.country_id.code

        if l10n_code and l10n_code in supported_l10n.keys():
            return request.redirect(f"/app/localization-{supported_l10n[l10n_code]}")

        # Fallback if no corresponding l10n found
        return request.redirect("/app/localization-all")

    @route(['/security-advisories', '/security-advisories/<path>.txt'], website=True, auth='public', type='http', sitemap=True, readonly=True)
    def security_advisory(self, path=None, **kw):
        if path and not request.session.security_advisory_code:
            return request.redirect("/security-advisories")
        if request.httprequest.method == 'POST' and kw.get("code"):
            code = kw["code"]
            contract = request.env['sale.order'].sudo()._get_most_valid_subscription(code)
            if contract and contract.enterprise_has_download:
                request.session.security_advisory_code = code
                return request.redirect("/security-advisories")
            return request.render("openerp_website.security_advisories", {"error": "Invalid Enterprise Code"})

        cve_dir = tools.file_path("openerp_website/data/advisories")
        if not path:
            cve_files = [f for f in os.listdir(cve_dir) if f.startswith("CVE-")]
            values = {"cve_files": cve_files, "code": request.session.security_advisory_code}
            return request.render("openerp_website.security_advisories", values)
        if path:
            try:
                # allowed format: CVE-YEAR-ID
                cve_id = int(path.rsplit("-", 1)[-1])
                cve_stream = http.Stream.from_path(f"{cve_dir}/CVE-2021-{cve_id}.txt")
                cve_stream.mimetype="text/plain"
                return cve_stream.get_response()
            except Exception:
                return werkzeug.exceptions.NotFound()

    def _make_watermark_doc(self, text, pagesize=None):
        f = io.BytesIO()
        page = canvas.Canvas(f, pagesize=pagesize or (21 * cm, 29.7 * cm))
        page.translate(7 * cm, 12 * cm)
        page.rotate(45)
        page.setFont("Helvetica", 40)
        page.setFillColorRGB(0.55, 0.55, 0.55)
        page.setFillAlpha(0.3)
        page.drawString(0, 0, text)
        page.save()
        f.flush()
        return f

    def _generate_watermarked_pdf(self, file_path, watermark):
        """Generate a PDF file with an added watermark overlay (in-memory!).

            :param str infile: a path to a PDF file within addons-path!
            :param str watermark: the watermark text, should be 1-2 lines, not too long,
                                  no attempt is made to resize it.
            :rtype: bytes
        """
        with tools.file_open(file_path, "rb") as infile:
            watermark_doc = PdfFileReader(self._make_watermark_doc(watermark))
            watermark_page = watermark_doc.getPage(0)
            pdf_reader = PdfFileReader(infile)
            pdf_writer = PdfFileWriter()
            for page in range(pdf_reader.getNumPages()):
                page = pdf_reader.getPage(page)
                page.mergePage(watermark_page)
                page.compressContentStreams() # divides by 3 the final size!
                pdf_writer.addPage(page)
            out = io.BytesIO()
            pdf_writer.write(out)
            return out.getvalue()

    @route(['/security/compliance-reports', '/security/compliance-reports/<path>.pdf'],
           auth='public', type='http', website=True, sitemap=True)
    def compliance_reports(self, path=None, **kw):
        if not request.session.uid:
            # show a more friendly landing page than auth=user, as users will be landing here
            # after signing the NDA, and can't go back
            return request.render("openerp_website.compliance_reports")

        # user is logged in
        report_dir = tools.file_path("openerp_website/data/compliance_reports")

        # no path - dir listing
        if not path:
            files = sorted((f for f in os.listdir(report_dir) if f.endswith(".pdf")), reverse=True)
            values = {
                "files": [
                    {
                        # split filename based on convention: 2099-01-01-REPORT.pdf
                        "file": f,
                        "name": f[11:].replace(".pdf", ""),
                        "date": f[:10],
                        "year": f[:4],
                    }
                    for f in files
                ]
            }
            return request.render("openerp_website.compliance_reports", values)

        # path requested
        try:
            # extra safety fuse against path traversals - don't change this!
            assert re.match(r"^[A-Za-z0-9 _-]+$", path)
            fpath = f"{report_dir}/{path}.pdf"
            pdf_bytes = self._generate_watermarked_pdf(fpath, request.env.user.login)
            headers = {'content-type': "application/pdf", "content-disposition": f"attachment; filename=Odoo_{path}.pdf"}
            _logger.info(f"Compliance report download {path}.pdf for user {request.env.user.login}")
            return request.make_response(pdf_bytes, headers)
        except Exception:
            _logger.exception("failed")
            return werkzeug.exceptions.NotFound()

    @route('/scaleup-teacher', auth='public', methods=['GET', 'POST'], type='http', website=True, csrf=True, sitemap=True)
    def odoo_scaleup_teacher(self, **kwargs):
        thanks = partner = opp = False
        if request.httprequest.method == 'POST':
            # create record
            thanks = True
            partner_vals = {
                'company_name': kwargs['school_name'],
                'name': kwargs['name'],
                'email': kwargs['email'],
                'street': kwargs['street'],
                'zip': kwargs['zip'],
                'city': kwargs['city'],
                'phone': kwargs['phone'],
                'country_id': int(kwargs['country_id']),
            }
            if kwargs.get('state_id'):
                partner_vals['state_id'] = int(kwargs['state_id'])
            partner = request.env['res.partner'].sudo().create(partner_vals)

            lang = request.env.ref('base.lang_%s' % kwargs['game_lang']).id

            vals = {w: html2text.html2text(kwargs[w]) for w in 'school_name school_link course student_count game_lang comment'.split(' ')}
            description = """
                    School: {school_name}<br/>
                    Link: <a href='{school_link}'>{school_link}</a><br/>
                    Course: {course} <br/>
                    Student: {student_count} <br/>
                    Lang: {game_lang} <br/><br/>
                    Proof/Comment: {comment}
            """.format(**vals)  # school_link could contain hacky code but will be sanitized by Html sanitizer

            if kwargs['game_lang'] == 'es':  # temporary since we don't have Spanish game in Belgium
                team_id = request.env.ref('openerp_crm.crm_team_edu_mx').id
            else:
                team_id = request.env['crm.team'].get_edu_team(request.geoip.country_code).id

            values = {
                'type': 'opportunity',
                'name': 'ScaleUp: %s' % kwargs['school_name'],
                'partner_id': partner.id,
                'lang_id': lang,
                'user_id': False,  # avoid to have current logged user (public or customer) as saleperson
                'team_id': team_id,
                'tag_ids': [(4, request.env.ref('openerp_website.tag_edu_teacher_lead').sudo().id, False)],
                'description': description,
            }
            opp = request.env['crm.lead'].sudo().create(values)

            attachs = []
            for file in request.httprequest.files.getlist('proof_files'):
                attachs.append({
                    'name': file.filename,
                    'raw': file.read(),
                    'res_model': 'crm.lead',
                    'res_id': opp.id,
                })

            request.env['ir.attachment'].sudo().create(attachs)

        return request.render("openerp_website.scaleup_teacher", dict(thanks=thanks, partner=partner, lead=opp))
